PRAGMA user_version=1;

CREATE TABLE MSG_CONVERSATION_TABLE
(
	CONV_ID INTEGER PRIMARY KEY ,
	UNREAD_CNT INTEGER DEFAULT 0 ,
	SMS_CNT INTEGER DEFAULT 0 ,
	MMS_CNT INTEGER DEFAULT 0 ,
	MAIN_TYPE INTEGER DEFAULT 0 ,
	SUB_TYPE INTEGER DEFAULT 0 ,
	MSG_DIRECTION INTEGER DEFAULT 0 ,
	DISPLAY_TIME DATETIME ,
	DISPLAY_NAME TEXT NOT NULL DEFAULT '' ,
	MSG_TEXT TEXT NOT NULL DEFAULT '' ,
	TRANSLATION INTEGER DEFAULT 0,
	SENDER_LANG INTEGER DEFAULT 0,
	RECEIVER_LANG INTEGER DEFAULT 0,
	ONLY_RECEIVE INTEGER DEFAULT 0,
	INCLUDE_ORGMSG INTEGER DEFAULT 0,
	LAST_MSG_ID INTEGER DEFAULT 0
);

CREATE TABLE MSG_ADDRESS_TABLE
(
	ADDRESS_ID INTEGER PRIMARY KEY ,
	CONV_ID INTEGER  NOT NULL ,
	ADDRESS_TYPE INTEGER ,
	RECIPIENT_TYPE INTEGER ,
	ADDRESS_VAL TEXT NOT NULL DEFAULT '' ,
	CONTACT_ID INTEGER ,
	ADDR_BOOK_ID INTEGER ,
	FIRST_NAME TEXT NOT NULL DEFAULT '' ,
	LAST_NAME TEXT NOT NULL DEFAULT '' ,
	MIDDLE_NAME TEXT NOT NULL DEFAULT '' ,
	PREFIX TEXT NOT NULL DEFAULT '' ,
	SUFFIX TEXT NOT NULL DEFAULT '' ,
	IMAGE_PATH TEXT NOT NULL DEFAULT '' ,
	SYNC_TIME DATETIME ,

	FOREIGN KEY (CONV_ID) REFERENCES MSG_CONVERSATION_TABLE (CONV_ID)
);

CREATE TABLE MSG_FOLDER_TABLE
(
	FOLDER_ID INTEGER PRIMARY KEY ,
	FOLDER_NAME TEXT NOT NULL ,
	FOLDER_TYPE INTEGER DEFAULT 0
);

CREATE TABLE MSG_MESSAGE_TABLE
(
	MSG_ID INTEGER PRIMARY KEY ,
	CONV_ID INTEGER NOT NULL ,
	FOLDER_ID INTEGER NOT NULL ,
	STORAGE_ID INTEGER NOT NULL ,
	MAIN_TYPE INTEGER NOT NULL ,
	SUB_TYPE INTEGER NOT NULL ,
	DISPLAY_TIME DATETIME ,
	DATA_SIZE INTEGER DEFAULT 0 ,
	NETWORK_STATUS INTEGER DEFAULT 0 ,
	READ_STATUS INTEGER DEFAULT 0 ,
	PROTECTED INTEGER DEFAULT 0 ,
	PRIORITY INTEGER DEFAULT 0 ,
	MSG_DIRECTION INTEGER NOT NULL ,
	SCHEDULED_TIME DATETIME ,
	BACKUP INTEGER DEFAULT 0 ,
	SUBJECT TEXT NOT NULL DEFAULT '' ,
	MSG_DATA TEXT NOT NULL DEFAULT '' ,
	THUMB_PATH TEXT NOT NULL DEFAULT '' ,
	MSG_TEXT TEXT NOT NULL DEFAULT '' ,
	ATTACHMENT_COUNT INTEGER DEFAULT 0 ,

	FOREIGN KEY (CONV_ID) REFERENCES MSG_CONVERSATION_TABLE (CONV_ID) ,
	FOREIGN KEY (FOLDER_ID) REFERENCES MSG_FOLDER_TABLE (FOLDER_ID)
);

CREATE TABLE MSG_SIM_TABLE
(
	SIM_ID INTEGER PRIMARY KEY ,
	MSG_ID INTEGER NOT NULL ,

	FOREIGN KEY(MSG_ID) REFERENCES MSG_MESSAGE_TABLE(MSG_ID)
);

CREATE TABLE MSG_PUSH_TABLE
(
	MSG_ID INTEGER PRIMARY KEY ,
	ACTION INTEGER ,
	CREATED INTEGER ,
	EXPIRES INTEGER ,
	ID TEXT NOT NULL DEFAULT '' ,
	HREF TEXT NOT NULL DEFAULT '' ,
	CONTENT TEXT NOT NULL DEFAULT '' ,

	FOREIGN KEY(MSG_ID) REFERENCES MSG_MESSAGE_TABLE(MSG_ID)
);

CREATE TABLE MSG_CBMSG_TABLE
(
	MSG_ID INTEGER PRIMARY KEY ,
	CB_MSG_ID INTEGER NOT NULL ,

	FOREIGN KEY(MSG_ID) REFERENCES MSG_MESSAGE_TABLE(MSG_ID)
);

CREATE TABLE MSG_SYNCML_TABLE
(
	MSG_ID INTEGER PRIMARY KEY ,
	EXT_ID INTEGER NOT NULL ,
	PINCODE INTEGER NOT NULL ,

	FOREIGN KEY(MSG_ID) REFERENCES MSG_MESSAGE_TABLE(MSG_ID)
);

CREATE TABLE MSG_SCHEDULED_TABLE
(
	MSG_ID INTEGER PRIMARY KEY ,
	ALARM_ID INTEGER NOT NULL ,

	FOREIGN KEY(MSG_ID) REFERENCES MSG_MESSAGE_TABLE(MSG_ID)
);

CREATE TABLE MSG_SMS_SENDOPT_TABLE
(
	MSG_ID INTEGER PRIMARY KEY ,
	DELREP_REQ INTEGER NOT NULL ,
	KEEP_COPY INTEGER NOT NULL ,
	REPLY_PATH INTEGER NOT NULL ,
	ENCODE_TYPE INTEGER NOT NULL ,

	FOREIGN KEY(MSG_ID) REFERENCES MSG_MESSAGE_TABLE(MSG_ID)
);

CREATE TABLE MSG_FILTER_TABLE
(
	FILTER_ID INTEGER PRIMARY KEY ,
	FILTER_TYPE INTEGER NOT NULL ,
	FILTER_VALUE TEXT NOT NULL ,
	FILTER_ACTIVE INTEGER DEFAULT 0
);

CREATE TABLE MSG_MMS_MESSAGE_TABLE
(
	MSG_ID INTEGER PRIMARY KEY ,
	TRANSACTION_ID TEXT NOT NULL DEFAULT '' ,
	MESSAGE_ID TEXT NOT NULL DEFAULT '' ,
	FWD_MESSAGE_ID TEXT NOT NULL DEFAULT '' ,
	CONTENTS_LOCATION TEXT NOT NULL DEFAULT '' ,
	FILE_PATH TEXT NOT NULL DEFAULT '' ,
	VERSION INTEGER NOT NULL ,
	DATA_TYPE INTEGER DEFAULT -1 ,
	DATE DATETIME ,
	HIDE_ADDRESS INTEGER DEFAULT 0 ,
	ASK_DELIVERY_REPORT INTEGER DEFAULT 0 ,
	REPORT_ALLOWED INTEGER DEFAULT 0 ,
	READ_REPORT_ALLOWED_TYPE INTEGER DEFAULT 0 ,
	ASK_READ_REPLY INTEGER DEFAULT 0 ,
	READ INTEGER DEFAULT 0 ,
	READ_REPORT_SEND_STATUS INTEGER DEFAULT 0 ,
	READ_REPORT_SENT INTEGER DEFAULT 0 ,
	PRIORITY INTEGER DEFAULT 0 ,
	KEEP_COPY INTEGER DEFAULT 0 ,
	MSG_SIZE INTEGER NOT NULL ,
	MSG_CLASS INTEGER DEFAULT -1 ,
	EXPIRY_TIME DATETIME ,
	CUSTOM_DELIVERY_TIME INTEGER DEFAULT 0 ,
	DELIVERY_TIME DATETIME ,
	MSG_STATUS INTEGER DEFAULT -1 ,

	FOREIGN KEY(MSG_ID) REFERENCES MSG_MESSAGE_TABLE(MSG_ID)
);

CREATE TABLE MSG_MMS_PREVIEW_INFO_TABLE
(
	MSG_ID INTEGER NOT NULL ,
	TYPE INTEGER,
	VALUE TEXT,
	COUNT INTEGER,

	FOREIGN KEY(MSG_ID) REFERENCES MSG_MESSAGE_TABLE(MSG_ID)
);

CREATE TABLE MSG_SMS_REPORT_TABLE
(
	MSG_ID INTEGER NOT NULL ,
	ADDRESS_VAL TEXT NOT NULL DEFAULT '' ,
	MSG_REF INTEGER ,
	STATUS_TYPE INTEGER ,
	STATUS INTEGER DEFAULT 0 ,
	TIME DATETIME
);

CREATE TABLE MSG_REPORT_TABLE
(
	MSG_ID INTEGER NOT NULL ,
	ADDRESS_VAL TEXT NOT NULL DEFAULT '' ,
	STATUS_TYPE INTEGER ,
	STATUS INTEGER DEFAULT 0 ,
	TIME DATETIME
);

CREATE TABLE MSG_PUSHCFG_TABLE
(
	PUSH_ID INTEGER PRIMARY KEY ,
	CONTENT_TYPE TEXT NOT NULL DEFAULT '' ,
	APP_ID TEXT NOT NULL DEFAULT '' ,
	PKG_NAME TEXT NOT NULL DEFAULT '' ,
	LAUNCH INTEGER,
	APPCODE INTEGER,
	SECURE INTEGER
);

CREATE TABLE MSG_TMP_MSGID_TABLE
(
	MSG_ID INTEGER
);

CREATE TABLE MSG_CB_CHANNEL_INFO_TABLE
(
	CHANNEL_ID INTEGER PRIMARY KEY ,
	CHANNEL_ACTIVATION INTEGER DEFAULT 0,
	CHANNEL_FROM INTEGER DEFAULT 0,
	CHANNEL_TO INTEGER DEFAULT 0,
	CHANNEL_NAME TEXT NOT NULL DEFAULT ''
);

CREATE TABLE MSG_MULTIPART_TABLE
(
	_ID INTEGER PRIMARY KEY AUTOINCREMENT,
	MSG_ID INTEGER NOT NULL ,
	SEQ INTEGER DEFAULT 0,
	CONTENT_TYPE TEXT,
	NAME TEXT,
	CHARSET INTEGER,
	CONTENT_ID TEXT,
	CONTENT_LOCATION TEXT,
	FILE_PATH TEXT,
	TEXT TEXT,
	TCS_LEVEL INTEGER DEFAULT -1,
	MALWARE_ALLOW INTEGER DEFAULT 0,
	THUMB_FILE_PATH TEXT,

	FOREIGN KEY(MSG_ID) REFERENCES MSG_MESSAGE_TABLE(MSG_ID)
);

CREATE TABLE MSG_CDMA_CB_CHANNEL_INFO_TABLE
(
	CHANNEL_ID INTEGER PRIMARY KEY,
	CHANNEL_ACTIVATION INTEGER DEFAULT 0,
	CHANNEL_CATEGORY INTEGER DEFAULT 0,
	CHANNEL_LANGUAGE INTEGER DEFAULT 0,
	CHANNEL_NAME TEXT NOT NULL DEFAULT ''
);

CREATE TABLE MSG_UNIQUENESS_INFO_TABLE
(
	MSG_ID INTEGER NOT NULL,
	TELE_MSG_ID INTEGER NOT NULL,
	ADDRESS TEXT DEFAULT '',
	SUB_ADDRESS TEXT DEFAULT '',
	TIME_STAMP TEXT DEFAULT '',
	TELESVC_ID INTEGER NOT NULL,

	FOREIGN KEY(MSG_ID) REFERENCES MSG_MESSAGE_TABLE(MSG_ID)
);

CREATE TABLE MSG_ADDRESS_TEMP_TABLE
(
	ADDRESS_VAL TEXT NOT NULL DEFAULT ''
);

CREATE INDEX MSG_CONVERSATION_INDEX ON MSG_CONVERSATION_TABLE(CONV_ID);
CREATE INDEX MSG_FOLDER_INDEX ON MSG_FOLDER_TABLE(FOLDER_ID);
CREATE INDEX MSG_MESSAGE_INDEX ON MSG_MESSAGE_TABLE(MSG_ID, CONV_ID, FOLDER_ID);

INSERT INTO MSG_FOLDER_TABLE VALUES (1, 'INBOX', 1);
INSERT INTO MSG_FOLDER_TABLE VALUES (2, 'OUTBOX', 2);
INSERT INTO MSG_FOLDER_TABLE VALUES (3, 'SENTBOX', 2);
INSERT INTO MSG_FOLDER_TABLE VALUES (4, 'DRAFT', 3);
INSERT INTO MSG_FOLDER_TABLE VALUES (5, 'CBMSGBOX', 1);
INSERT INTO MSG_FOLDER_TABLE VALUES (6, 'SPAMBOX', 4);

INSERT INTO MSG_PUSHCFG_TABLE VALUES (1, 'text/vnd.wap.si', 'X-Wap-Application-Id: x-wap-application:wml.ua', '', 0, 1, 0);
INSERT INTO MSG_PUSHCFG_TABLE VALUES (2, 'application/vnd.wap.sic', 'X-Wap-Application-Id: x-wap-application:wml.ua', '', 0, 2, 0);
INSERT INTO MSG_PUSHCFG_TABLE VALUES (3, 'text/vnd.wap.sl', 'X-Wap-Application-Id: x-wap-application:wml.ua', '', 0, 3, 0);
INSERT INTO MSG_PUSHCFG_TABLE VALUES (4, 'application/vnd.wap.slc', 'X-Wap-Application-Id: x-wap-application:wml.ua', '', 0, 4, 0);
INSERT INTO MSG_PUSHCFG_TABLE VALUES (5, 'text/vnd.wap.co', 'X-Wap-Application-Id: x-wap-application:wml.ua', '', 0, 5, 0);

INSERT INTO MSG_PUSHCFG_TABLE VALUES (6, 'application/vnd.wap.coc', 'X-Wap-Application-Id: x-wap-application:wml.ua', '', 0, 6, 0);

INSERT INTO MSG_PUSHCFG_TABLE VALUES (7, 'application/vnd.wap.mms-message', 'X-Wap-Application-Id: x-wap-application:mms.ua', '', 0, 7, 0);
INSERT INTO MSG_PUSHCFG_TABLE VALUES (8, 'application/vnd.wap.sia', 'X-Wap-Application-Id: x-wap-application:push.sia', '', 0, 8, 0);
INSERT INTO MSG_PUSHCFG_TABLE VALUES (9, 'application/vnd.syncml.dm+wbxml', 'X-Wap-Application-Id: x-wap-application:push.syncml.dm', '', 0, 9, 0);
INSERT INTO MSG_PUSHCFG_TABLE VALUES (10, 'application/vnd.syncml.dm+xml', 'X-Wap-Application-Id: x-wap-application:push.syncml.dm', '', 0, 10, 0);

INSERT INTO MSG_PUSHCFG_TABLE VALUES (11, 'application/vnd.syncml.notification', 'X-Wap-Application-Id: x-wap-application:push.syncml.dm', '', 0, 11, 0);
INSERT INTO MSG_PUSHCFG_TABLE VALUES (12, 'application/vnd.syncml.ds.notification', 'X-Wap-Application-Id: x-wap-application:push.syncml.ds', '', 0, 12, 0);
INSERT INTO MSG_PUSHCFG_TABLE VALUES (13, 'application/vnd.syncml+wbxml', 'X-Wap-Application-Id:x-wap-application:push.syncml', '', 0, 13, 0);
INSERT INTO MSG_PUSHCFG_TABLE VALUES (14, 'application/vnd.wap.locc+wbxml', 'X-Wap-Application-Id: x-wap-application:loc.ua', '', 0, 14, 0);
INSERT INTO MSG_PUSHCFG_TABLE VALUES (15, 'application/vnd.wap.loc+xml', 'X-Wap-Application-Id: x-wap-application:loc.ua', '', 0, 15, 0);

INSERT INTO MSG_PUSHCFG_TABLE VALUES (16, 'application/vnd.oma.dd+xml', 'X-Wap-Application-Id: x-wap-application:loc.ua', '', 0, 16, 0);
INSERT INTO MSG_PUSHCFG_TABLE VALUES (17, 'application/vnd.oma.drm.message', 'X-Wap-Application-Id: x-wap-application:drm.ua', '', 0, 17, 0);
INSERT INTO MSG_PUSHCFG_TABLE VALUES (18, 'application/vnd.oma.drm.content', 'X-Wap-Application-Id: x-wap-application:drm.ua', '', 0, 18, 0);
INSERT INTO MSG_PUSHCFG_TABLE VALUES (19, 'application/vnd.oma.drm.rights+xml', 'X-Wap-Application-Id: x-wap-application:drm.ua', '', 0, 19, 0);
INSERT INTO MSG_PUSHCFG_TABLE VALUES (20, 'application/vnd.oma.drm.rights+wbxml', 'X-Wap-Application-Id: x-wap-application:drm.ua', '', 0, 20, 0);

INSERT INTO MSG_PUSHCFG_TABLE VALUES (21, 'application/vnd.oma.drm.ro+xml', 'X-Wap-Application-Id: x-wap-application:drm.ua', '', 0, 21, 0);
INSERT INTO MSG_PUSHCFG_TABLE VALUES (22, 'application/vnd.oma.drm.roap-pdu+xml', 'X-Wap-Application-Id: x-wap-application:drm.ua', '', 0, 22, 0);
INSERT INTO MSG_PUSHCFG_TABLE VALUES (23, 'application/vnd.oma.drm.roap-trigger+xml', 'X-Wap-Application-Id: x-wap-application:drm.ua', '', 0, 23, 0);
INSERT INTO MSG_PUSHCFG_TABLE VALUES (24, 'application/vnd.oma.drm.roap-trigger+wbxml', 'X-Wap-Application-Id: x-wap-application:drm.ua', '', 0, 24, 0);
INSERT INTO MSG_PUSHCFG_TABLE VALUES (25, 'text/vnd.wap.connectivity-xml', 'X-Wap-Application-Id: x-wap-application:drm.ua', '', 0, 26, 0);

INSERT INTO MSG_PUSHCFG_TABLE VALUES (26, 'application/vnd.wap.connectivity-wbxml', 'X-Wap-Application-Id: x-wap-samsung:provisioning.ua', '', 0, 27, 0);
INSERT INTO MSG_PUSHCFG_TABLE VALUES (27, 'application/x-wap-prov.browser-settings', 'X-Wap-Application-Id: x-wap-samsung:provisioning.ua', '', 0, 28, 0);
INSERT INTO MSG_PUSHCFG_TABLE VALUES (28, 'application/x-wap-prov.browser-bookmarks', 'X-Wap-Application-Id: x-wap-samsung:provisioning.ua', '', 0, 29, 0);
INSERT INTO MSG_PUSHCFG_TABLE VALUES (29, 'application/x-wap-prov.syncset+xml', 'X-Wap-Application-Id: x-wap-samsung:provisioning.ua', '', 0, 30, 0);
INSERT INTO MSG_PUSHCFG_TABLE VALUES (30, 'application/x-wap-prov.syncset+wbxml', 'X-Wap-Application-Id: x-wap-samsung:provisioning.ua', '', 0, 31, 0);

INSERT INTO MSG_PUSHCFG_TABLE VALUES (33, 'application/vnd.wv.csp.cir', 'X-Wap-Application-Id: x-wap-application:wv.ua', '', 0, 34, 0);
INSERT INTO MSG_PUSHCFG_TABLE VALUES (34, 'application/vnd.omaloc-supl-init', 'X-Wap-Application-Id: x-oma-application:ulp.ua', '', 0, 44, 0);

INSERT INTO MSG_CDMA_CB_CHANNEL_INFO_TABLE VALUES (0, 1, 4096, 1, 'Presidential');
INSERT INTO MSG_CDMA_CB_CHANNEL_INFO_TABLE VALUES (1, 1, 4097, 1, 'Extreme');
INSERT INTO MSG_CDMA_CB_CHANNEL_INFO_TABLE VALUES (2, 1, 4098, 1, 'Severe');
INSERT INTO MSG_CDMA_CB_CHANNEL_INFO_TABLE VALUES (3, 1, 4099, 1, 'Amber');
INSERT INTO MSG_CDMA_CB_CHANNEL_INFO_TABLE VALUES (4, 0, 4100, 1, 'Test');
