/*
 * Copyright (c) 2014 Samsung Electronics Co., Ltd. All rights reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
*/

/*==================================================================================================
                                         INCLUDE FILES
==================================================================================================*/
#include "MsgDebug.h"
#include "MsgStorageHandler.h"
#include "MsgUtilFunction.h"
#include "MsgDPMUtils.h"

#include <dpm/restriction.h>

device_policy_manager_h dpm_handle = NULL;
int dpm_sms_callback_id = 0;
int dpm_mms_callback_id = 0;

/*==================================================================================================
                                     FUNCTION IMPLEMENTATION
==================================================================================================*/

static void dpm_sms_policy_changed_callback(const char* name, const char* object, void *user_data)
{
	MSG_INFO("dpm_sms_policy_changed_callback called. name [%s] object [%s]", name, object);

	int ret = 0;
	int state = 0;

	ret = dpm_restriction_get_messaging_state(dpm_handle, "sim1", &state);
	if (ret != DPM_ERROR_NONE) {
		MSG_ERR("dpm_restriction_get_messaging_state failed [%d]", ret);
		return;
	}

	msg_set_dpm_policy(MSG_SMS_TYPE, state);

	if (state == 1) {
		MsgStoUpdateDPMRestrictedStatus(MSG_SMS_TYPE);
		/* temporal */
		MsgStoUpdateDPMRestrictedStatus(MSG_MMS_TYPE);
	}
}

#if 0
static void dpm_mms_policy_changed_callback(const char* name, const char* object, void *user_data)
{
	MSG_INFO("dpm_mms_policy_changed_callback called. name [%s] object [%s]", name, object);

	int ret = 0;
	int state = 0;

	ret = dpm_restriction_get_messaging_state(dpm_handle, &dpm_policy_enable[MSG_MMS_TYPE]);
	if (ret != DPM_ERROR_NONE) {
		MSG_ERR("dpm_restriction_get_messaging_state failed [%d]", ret);
		return;
	}

	msg_set_dpm_policy(MSG_MMS_TYPE, state);

	if (state == 1)
		MsgStoUpdateDPMRestrictedStatus(MSG_MMS_TYPE);
}
#endif

void __get_dpm_policy()
{
	int ret = 0;
	int state = 0;

	ret = dpm_restriction_get_messaging_state(dpm_handle, "sim1", &state);
	if (ret != DPM_ERROR_NONE) {
		MSG_ERR("dpm_restriction_get_messaging_state failed [%d]", ret);
		return;
	}

	MSG_DEBUG("sms policy [%d]", state);
	msg_set_dpm_policy(MSG_SMS_TYPE, state);
//	MSG_DEBUG("mms policy [%d]", dpm_policy_enable[MSG_MMS_TYPE]);

	ret = dpm_add_policy_changed_cb(dpm_handle, "messaging", dpm_sms_policy_changed_callback, NULL, &dpm_sms_callback_id);
	if (ret != DPM_ERROR_NONE) {
		MSG_ERR("dpm_add_policy_changed_cb failed [%d]", ret);
		return;
	}
}

void msg_init_dpm_policy()
{
	MSG_BEGIN();

	int cnt = 5;
	while (cnt--) {
		dpm_handle = dpm_manager_create();
		if (dpm_handle == NULL) {
			MSG_ERR("dpm_manager_create() failed");
		} else {
			MSG_DEBUG("dpm_manager_create() success");
			__get_dpm_policy();
			break;
		}

		sleep(1);
	}

	MSG_END();
}


void msg_deinit_dpm_policy()
{
	if (dpm_sms_callback_id)
		dpm_remove_policy_changed_cb(dpm_handle, dpm_sms_callback_id);

	if (dpm_mms_callback_id)
		dpm_remove_policy_changed_cb(dpm_handle, dpm_mms_callback_id);

	if (dpm_handle)
		dpm_manager_destroy(dpm_handle);
}
