/*
 * Copyright (c) 2014 Samsung Electronics Co., Ltd. All rights reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
*/

#ifndef MSG_CMD_TYPES_H
#define MSG_CMD_TYPES_H

/*==================================================================================================
											INCLUDE FILES
==================================================================================================*/
#include "MsgTypes.h"


/*==================================================================================================
											DEFINES
==================================================================================================*/
#define MAX_COOKIE_LEN 20


/*==================================================================================================
											TYPES
==================================================================================================*/
typedef unsigned int MSG_CMD_TYPE_T;

typedef unsigned int MSG_EVENT_TYPE_T;


/*==================================================================================================
											STRUCTURES
==================================================================================================*/
typedef struct _MSG_CMD_S {
	MSG_CMD_TYPE_T	cmdType;
	char				cmdCookie[MAX_COOKIE_LEN];
	char				cmdData[2];
} MSG_CMD_S;


typedef struct _MSG_EVENT_S {
	MSG_EVENT_TYPE_T		eventType;
	msg_error_t			result;
	char					data[2];
} MSG_EVENT_S;


/*==================================================================================================
											ENUMS
==================================================================================================*/

enum _MSG_CMD_TYPE_E {
	MSG_CMD_OPEN_HANDLE = 0,
	MSG_CMD_CLOSE_HANDLE,
	MSG_CMD_GET_STORAGELIST,
	MSG_CMD_ADD_MSG,
	MSG_CMD_ADD_SYNCML_MSG,
/* 5 */
	MSG_CMD_UPDATE_MSG,
	MSG_CMD_UPDATE_READ,
	MSG_CMD_UPDATE_PROTECTED,
	MSG_CMD_DELETE_MSG,
	MSG_CMD_DELALL_MSGINFOLDER,
/* 10 */
	MSG_CMD_MOVE_MSGTOFOLDER,
	MSG_CMD_MOVE_MSGTOSTORAGE,
	MSG_CMD_COUNT_MSG,
	MSG_CMD_GET_MSG,
	MSG_CMD_ADD_FOLDER,
/* 15 */
	MSG_CMD_UPDATE_FOLDER,
	MSG_CMD_DELETE_FOLDER,
	MSG_CMD_GET_FOLDERLIST,
	MSG_CMD_ADD_FILTER,
	MSG_CMD_UPDATE_FILTER,
/* 20 */
	MSG_CMD_DELETE_FILTER,
	MSG_CMD_GET_FILTERLIST,
	MSG_CMD_SET_FILTER_OPERATION,
	MSG_CMD_GET_FILTER_OPERATION,
	MSG_CMD_SUBMIT_REQ,
/* 25 */
	MSG_CMD_REG_SENT_STATUS_CB,
	MSG_CMD_REG_STORAGE_CHANGE_CB,
	MSG_CMD_REG_INCOMING_MSG_CB,
	MSG_CMD_REG_INCOMING_MMS_CONF_MSG_CB,
	MSG_CMD_REG_INCOMING_SYNCML_MSG_CB,
/* 30 */
	MSG_CMD_REG_INCOMING_LBS_MSG_CB,
	MSG_CMD_PLG_SENT_STATUS_CNF,
	MSG_CMD_PLG_STORAGE_CHANGE_IND,
	MSG_CMD_PLG_INCOMING_MSG_IND,
	MSG_CMD_PLG_INCOMING_MMS_CONF,
/* 35 */
	MSG_CMD_PLG_INCOMING_SYNCML_IND,
	MSG_CMD_PLG_INCOMING_LBS_IND,
	MSG_CMD_PLG_INIT_SIM_BY_SAT,
	MSG_CMD_GET_THREADVIEWLIST,
	MSG_CMD_DELETE_THREADMESSAGELIST,
/* 40 */
	MSG_CMD_GET_CONTACT_COUNT,
	MSG_CMD_GET_QUICKPANEL_DATA,
	MSG_CMD_COUNT_BY_MSGTYPE,
	MSG_CMD_RESET_DB,
	MSG_CMD_GET_MEMSIZE,
/* 45 */
	MSG_CMD_BACKUP_MESSAGE,
	MSG_CMD_RESTORE_MESSAGE,
	MSG_CMD_UPDATE_THREAD_READ,
	MSG_CMD_REG_SYNCML_MSG_OPERATION_CB,
	MSG_CMD_SYNCML_OPERATION,
/* 50 */
	MSG_CMD_GET_REPORT_STATUS,
	MSG_CMD_GET_THREAD_ID_BY_ADDRESS,
	MSG_CMD_GET_THREAD_INFO,
	MSG_CMD_GET_SMSC_OPT,
	MSG_CMD_GET_CB_OPT,
/* 55 */
	MSG_CMD_GET_SMS_SEND_OPT,
	MSG_CMD_GET_MMS_SEND_OPT,
	MSG_CMD_GET_MMS_RECV_OPT,
	MSG_CMD_GET_PUSH_MSG_OPT,
	MSG_CMD_GET_VOICE_MSG_OPT,
/* 60 */
	MSG_CMD_GET_GENERAL_MSG_OPT,
	MSG_CMD_GET_MSG_SIZE_OPT,
	MSG_CMD_SET_SMSC_OPT,
	MSG_CMD_SET_CB_OPT,
	MSG_CMD_SET_SMS_SEND_OPT,
/* 65 */
	MSG_CMD_SET_MMS_SEND_OPT,
	MSG_CMD_SET_MMS_RECV_OPT,
	MSG_CMD_SET_PUSH_MSG_OPT,
	MSG_CMD_SET_VOICE_MSG_OPT,
	MSG_CMD_SET_GENERAL_MSG_OPT,
/* 70 */
	MSG_CMD_SET_MSG_SIZE_OPT,
	MSG_CMD_REG_INCOMING_PUSH_MSG_CB,
	MSG_CMD_PLG_INCOMING_PUSH_IND,
	MSG_CMD_REG_INCOMING_CB_MSG_CB,
	MSG_CMD_PLG_INCOMING_CB_IND,
/* 75 */
	MSG_CMD_ADD_PUSH_EVENT,
	MSG_CMD_DELETE_PUSH_EVENT,
	MSG_CMD_UPDATE_PUSH_EVENT,
	MSG_CMD_DELETE_MESSAGE_BY_LIST,
	MSG_CMD_SET_FILTER_ACTIVATION,
/* 80 */
	MSG_CMD_ADD_SIM_MSG,
	MSG_CMD_PLG_RESEND_MESSAGE,
	MSG_CMD_REG_REPORT_MSG_INCOMING_CB,
	MSG_CMD_UPDATE_IMSI,
	MSG_CMD_SET_TEMP_ADDRESS_TABLE,

#ifdef FEATURE_SMS_CDMA
	MSG_CMD_PLG_CHECK_UNIQUENESS,
#endif

	MSG_CMD_ALLOW_TCS_MESSAGE,
	MSG_CMD_CHECK_PERMISSION,
	MSG_CMD_REG_THREAD_CHANGE_CB,
	MSG_CMD_PLG_THREAD_CHANGE_IND,

/* end of MSG_CMD; new CMD should be defined before MSG_CMD_NUM */
	MSG_CMD_NUM
};

enum _MSG_EVENT_TYPE_E {
	MSG_EVENT_OPEN_HANDLE = 0,
	MSG_EVENT_CLOSE_HANDLE,
	MSG_EVENT_GET_STORAGELIST,
	MSG_EVENT_ADD_MSG,
	MSG_EVENT_ADD_SYNCML_MSG,
/* 5 */
	MSG_EVENT_UPDATE_MSG,
	MSG_EVENT_UPDATE_READ,
	MSG_EVENT_UPDATE_PROTECTED,
	MSG_EVENT_DELETE_MSG,
	MSG_EVENT_DELALL_MSGINFOLDER,
/* 10 */
	MSG_EVENT_MOVE_MSGTOFOLDER,
	MSG_EVENT_MOVE_MSGTOSTORAGE,
	MSG_EVENT_COUNT_MSG,
	MSG_EVENT_GET_MSG,
	MSG_EVENT_ADD_FOLDER,
/* 15 */
	MSG_EVENT_UPDATE_FOLDER,
	MSG_EVENT_DELETE_FOLDER,
	MSG_EVENT_GET_FOLDERLIST,
	MSG_EVENT_ADD_FILTER,
	MSG_EVENT_UPDATE_FILTER,
/* 20 */
	MSG_EVENT_DELETE_FILTER,
	MSG_EVENT_GET_FILTERLIST,
	MSG_EVENT_SET_FILTER_OPERATION,
	MSG_EVENT_GET_FILTER_OPERATION,
	MSG_EVENT_SUBMIT_REQ,
/* 25 */
	MSG_EVENT_REG_SENT_STATUS_CB,
	MSG_EVENT_REG_STORAGE_CHANGE_CB,
	MSG_EVENT_REG_INCOMING_MSG_CB,
	MSG_EVENT_REG_INCOMING_MMS_CONF_MSG_CB,
	MSG_EVENT_REG_INCOMING_SYNCML_MSG_CB,
/* 30 */
	MSG_EVENT_REG_INCOMING_LBS_MSG_CB,
	MSG_EVENT_PLG_SENT_STATUS_CNF,
	MSG_EVENT_PLG_STORAGE_CHANGE_IND,
	MSG_EVENT_PLG_INCOMING_MSG_IND,
	MSG_EVENT_PLG_INCOMING_MMS_CONF,
/* 35 */
	MSG_EVENT_PLG_INCOMING_SYNCML_MSG_IND,
	MSG_EVENT_PLG_INCOMING_LBS_MSG_IND,
	MSG_EVENT_PLG_INIT_SIM_BY_SAT,
	MSG_EVENT_GET_THREADVIEWLIST,
	MSG_EVENT_DELETE_THREADMESSAGELIST,
/* 40 */
	MSG_EVENT_GET_CONTACT_COUNT,
	MSG_EVENT_GET_QUICKPANEL_DATA,
	MSG_EVENT_COUNT_BY_MSGTYPE,
	MSG_EVENT_RESET_DB,
	MSG_EVENT_GET_MEMSIZE,
/* 45 */
	MSG_EVENT_BACKUP_MESSAGE,
	MSG_EVENT_RESTORE_MESSAGE,
	MSG_EVENT_UPDATE_THREAD_READ,
	MSG_EVENT_REG_SYNCML_MSG_OPERATION_CB,
	MSG_EVENT_SYNCML_OPERATION,
/* 50 */
	MSG_EVENT_GET_REPORT_STATUS,
	MSG_EVENT_GET_THREAD_ID_BY_ADDRESS,
	MSG_EVENT_GET_THREAD_INFO,
	MSG_EVENT_GET_SMSC_OPT,
	MSG_EVENT_GET_CB_OPT,
/* 55 */
	MSG_EVENT_GET_SMS_SEND_OPT,
	MSG_EVENT_GET_MMS_SEND_OPT,
	MSG_EVENT_GET_MMS_RECV_OPT,
	MSG_EVENT_GET_PUSH_MSG_OPT,
	MSG_EVENT_GET_VOICE_MSG_OPT,
/* 60 */
	MSG_EVENT_GET_GENERAL_MSG_OPT,
	MSG_EVENT_GET_MSG_SIZE_OPT,
	MSG_EVENT_SET_SMSC_OPT,
	MSG_EVENT_SET_CB_OPT,
	MSG_EVENT_SET_SMS_SEND_OPT,
/* 65 */
	MSG_EVENT_SET_MMS_SEND_OPT,
	MSG_EVENT_SET_MMS_RECV_OPT,
	MSG_EVENT_SET_PUSH_MSG_OPT,
	MSG_EVENT_SET_VOICE_MSG_OPT,
	MSG_EVENT_SET_GENERAL_MSG_OPT,
/* 70 */
	MSG_EVENT_SET_MSG_SIZE_OPT,
	MSG_EVENT_REG_INCOMING_PUSH_MSG_CB,
	MSG_EVENT_PLG_INCOMING_PUSH_MSG_IND,
	MSG_EVENT_REG_INCOMING_CB_MSG_CB,
	MSG_EVENT_PLG_INCOMING_CB_MSG_IND,
/* 75 */
	MSG_EVENT_ADD_PUSH_EVENT,
	MSG_EVENT_DELETE_PUSH_EVENT,
	MSG_EVENT_UPDATE_PUSH_EVENT,
	MSG_EVENT_DELETE_MESSAGE_BY_LIST,
	MSG_EVENT_SET_FILTER_ACTIVATION,
/* 80 */
	MSG_EVENT_ADD_SIM_MSG,
	MSG_EVENT_PLG_RESEND_MESSAGE,
	MSG_EVENT_REG_REPORT_MSG_INCOMING_CB,
	MSG_EVENT_UPDATE_IMSI,
	MSG_EVENT_SET_TEMP_ADDRESS_TABLE,

#ifdef FEATURE_SMS_CDMA
	MSG_EVENT_PLG_CHECK_UNIQUENESS,
#endif
	MSG_EVENT_ALLOW_TCS_MESSAGE,
	MSG_EVENT_CHECK_PERMISSION,
	MSG_EVENT_REG_THREAD_CHANGE_CB,
	MSG_EVENT_PLG_THREAD_CHANGE_IND,

/* Enums that does not match _MSG_CMD_TYPE_E */
	MSG_EVENT_PLG_REPORT_MSG_INCOMING_IND,

/* end of MSG_EVENT; new EVENT should be defined before MSG_EVENT_NUM */
	MSG_EVENT_NUM
};



#endif /* MSG_CMD_TYPES_H */

