/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd. All rights reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
*/

#ifndef __MSG_MGR_PROFILE_H__
#define __MSG_MGR_PROFILE_H__

#include <stdlib.h>
#include <system_info.h>

typedef enum {
	_PROFILE_UNKNOWN = 0,
	_PROFILE_MOBILE = 0x1,
	_PROFILE_WEARABLE = 0x2,
	_PROFILE_TV = 0x4,
	_PROFILE_IVI = 0x8,
	_PROFILE_COMMON = 0x10,
} tizen_profile_t;

/* For optimization, make this extern and define in a shared C file */
extern tizen_profile_t profile;

/* Accessing system info */
int system_info_get_platform_string(const char *key, char **value);

static inline tizen_profile_t get_tizen_profile()
{
	char *profileName = NULL;

	if (__builtin_expect(profile != _PROFILE_UNKNOWN, 1))
		return profile;

	system_info_get_platform_string("http://tizen.org/feature/profile", &profileName);

	/* To pass the checking of g_ir */
	if (!profileName)
		return _PROFILE_UNKNOWN;

	switch (*profileName) {
	case 'm':
	case 'M':
		profile = _PROFILE_MOBILE;
		break;
	case 'w':
	case 'W':
		profile = _PROFILE_WEARABLE;
		break;
	case 't':
	case 'T':
		profile = _PROFILE_TV;
		break;
	case 'i':
	case 'I':
		profile = _PROFILE_IVI;
		break;
	default: // common or unknown ==> ALL ARE COMMON.
		profile = _PROFILE_COMMON;
	}
	free(profileName);

	return profile;
}
#endif /*__MSG_MGR_PROFILE_H__ */

